"""
Implementation common exceptions and data. (Python protocol)

This module should not be directly referenced as the
contents are imported into :py:mod:`traffic.lib` for access.

This module exists to eliminate any possibility of circular imports
given that the :py:mod:`traffic.lib` may import implementations.
"""
import collections

__all__ = [
	'polarity',
	'Exception',
	'Error',
	'ResourceError',
	'TerminatedError',
]

#: Tuple of polarity identifiers; :py:class:`traffic.abstract.Transit.polarity`.
polarity = collections.namedtuple("Polarity",
	("lose", "none", "gain"))(-1, 0, 1)

class Exception(Exception):
	"""
	Bottom Traffic Exception.
	"""

class Error(Exception):
	"""
	Generic Traffic Error.
	"""

class ResourceError(Error):
	"""
	Error raised when a Transit's resource is improperly managed.
	"""

class TerminatedError(Error):
	"""
	Error raised when a side-effect operation is performed on a Transit that is terminated.
	"""
